#include <stdlib.h>
#include <stdio.h>
#include <time.h>
#include <signal.h>
#include <curses.h>

#include "mini-mine.h"


static void new_game ();
static void game_over (char game_end);
static void cleanup (int sig);


int
main (int argc, char **argv)
{
  int key_pressed;
  char game_ends;

  /* initialize to default values */
  mine_field_xmax = XMAXDEF;
  mine_field_ymax = YMAXDEF;

  parseopts(argc, argv);

  /* setup curses cleanup in case of sigint */
  (void) signal (SIGINT, cleanup);
  /* setup curses screen */
  (void) initscr ();
  keypad (stdscr, TRUE);	/* enable keyboard mapping */
  (void) nonl ();		/* tell curses not to do NL->CR/NL on output */
  (void) cbreak ();
  (void) noecho ();

  new_game();

  while ((key_pressed = getch ()) != 'q')
    {
      switch (key_pressed)
	{
	case KEY_UP:
	  if (mine_field_cursor_y > 0)
	    {
	      mine_field_cursor (mine_field_cursor_x, mine_field_cursor_y, 0);
	      mine_field_cursor (mine_field_cursor_x, --mine_field_cursor_y,
				 1);
	    }
	  break;
	case KEY_DOWN:
	  if (mine_field_cursor_y < (mine_field_ymax - 1))
	    {
	      mine_field_cursor (mine_field_cursor_x, mine_field_cursor_y, 0);
	      mine_field_cursor (mine_field_cursor_x, ++mine_field_cursor_y,
				 1);
	    }
	  break;
	case KEY_LEFT:
	  if (mine_field_cursor_x > 0)
	    {
	      mine_field_cursor (mine_field_cursor_x, mine_field_cursor_y, 0);
	      mine_field_cursor (--mine_field_cursor_x, mine_field_cursor_y,
				 1);
	    }
	  break;
	case KEY_RIGHT:
	  if (mine_field_cursor_x < (mine_field_xmax - 1))
	    {
	      mine_field_cursor (mine_field_cursor_x, mine_field_cursor_y, 0);
	      mine_field_cursor (++mine_field_cursor_x, mine_field_cursor_y,
				 1);
	    }
	  break;
	case ' ':
	  game_ends = mine_field_pic (mine_field_cursor_x, mine_field_cursor_y);
	  if (game_ends)
	    game_over(game_ends);
	  break;
	case 'f':
	  mine_field_mark(mine_field_cursor_x,mine_field_cursor_y);
	  break;
	}
      if ((marks_true == mine_field_mines) && (marks_false == 0))
	game_over(WIN);
      /* we move the cursor to the active field, things are looking
	 nicer this way */
      move(mine_field_cursor_y+1,mine_field_cursor_x*2+1);
      refresh ();
    }

  cleanup (0);
  exit (0);
}


static void
new_game()
{
  (void) clear();
  initmf ();
  mine_field_clean();

  mine_field_cursor_x = CURSORX;
  mine_field_cursor_y = CURSORY;
  mine_field_cursor (mine_field_cursor_x, mine_field_cursor_y, 1);

  start_time = time((time_t *) NULL);
}


static void
game_over(char game_end)
{
  char time_msg[80];
  if (game_end == WIN)
    {
      (void) mvaddstr(12,1,"YOU ARE GREAT!");
      sprintf(time_msg, "You solved the puzzle in %i seconds!",
			 (int)(time((time_t *) NULL) - start_time));
      (void) mvaddstr(13,1,time_msg); 
    }
  else
    (void) mvaddstr(13,1,"GAME OVER!");
  (void) mvaddstr(14,1,"Try again? (y|n)");
  refresh ();
  if ((char)getch() == 'n')
    cleanup(0);
  else
    new_game();
}


static void
cleanup (int sig)
{
  endwin ();
  exit (0);
}
